#ifndef CoCoA_GlobalManager_H
#define CoCoA_GlobalManager_H

//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <memory>
// using std::auto_ptr

namespace CoCoA
{

  class ring;          // fwd decl -- defined in ring.H
  class FractionField; // fwd decl -- defined in FractionField.H

  // At most one instance of this class may exist at any given time.
  class GlobalManager
  {
  public:
    GlobalManager();
    ~GlobalManager();
  private: // disable copy ctor and assignment
    GlobalManager(const GlobalManager&);            ///< NEVER DEFINED -- disable copy ctor
    GlobalManager& operator=(const GlobalManager&); ///< NEVER DEFINED -- disable assignment
  private:
    static const GlobalManager* ourGlobalDataPtr; ///< NULL or addr of unique existing GlobalManager
    friend const ring& RingZ(); // defined in RingZ.C
    friend const FractionField& RingQ(); // defined in RingQ.C
  private: // data members
    std::auto_ptr<ring> myRingZPtr;
    std::auto_ptr<FractionField> myRingQPtr;
  };

} // end of namespace CoCoA



// RCS header/log in the next few lines.
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/GlobalManager.H,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: GlobalManager.H,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.2  2007/03/05 21:33:13  cocoa
// Improved/cleaned GlobalManager; added doc too.
//
// Revision 1.1  2007/03/03 14:02:11  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.1  2007/03/02 16:46:28  cocoa
// New foundations object which calls ctors and dtors of global objects.
//

#endif
