#!/bin/bash

# Script to generate automatically the unified header for CoCoALib.

# Check we have one arg.
if [ $# -ne 1 ]
then
  echo                                                                  >/dev/stderr
  echo "ERROR: $0 requires 1 arg (the version ID of the CoCoALibrary)." >/dev/stderr
  exit 1
fi
VERSION="$1"

# Check we're in the CoCoALib/include/CoCoA/ directory.  Give error if not.
CWD=`pwd`
LAST=`basename "$CWD"`
TMP=`dirname "$CWD"`
LASTBUTONE=`basename "$TMP"`
if [ "$LAST" != "CoCoA" -o "$LASTBUTONE" != "include" ]
then
  echo                                                                         >/dev/stderr
  echo "ERROR: $0 should be run only in the directory CoCoALib/include/CoCoA/" >/dev/stderr
  exit 1
fi

UNIFIEDHDR=library.H

# Move existing unified header into a back-up file (with .old suffix).
if [ -f "$UNIFIEDHDR" ]
then
  /bin/rm -rf "$UNIFIEDHDR.old"
  /bin/mv "$UNIFIEDHDR" "$UNIFIEDHDR.old"
fi

# Get names of the CoCoALib header files (before creating the new unified header file!)
# WARNING: the Makefile in the top CoCoALib directory looks for the version number using
#          a string search -- be careful if you change the format produced here!
COCOALIBHDRS=`/bin/ls *.H`
echo "#ifndef CoCoA_library_H"                                >> "$UNIFIEDHDR"
echo "#define CoCoA_library_H"                                >> "$UNIFIEDHDR"
echo                                                          >> "$UNIFIEDHDR"
echo "// Unified header file for CoCoALib (version $VERSION)" >> "$UNIFIEDHDR"
echo "// (produced automatically by MakeUnifiedHeader.sh)"    >> "$UNIFIEDHDR"
echo                                                          >> "$UNIFIEDHDR"
for file in $COCOALIBHDRS
do
  echo "#include \"CoCoA/$file\"" >> "$UNIFIEDHDR"
done
echo                                                          >> "$UNIFIEDHDR"
echo "#endif"                                                 >> "$UNIFIEDHDR"
