#ifndef CoCoA_ReductionCog_H
#define CoCoA_ReductionCog_H

//   Copyright (c)  2006  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.
//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "CoCoA/PPMonoid.H"
#include "CoCoA/SmartPtrIRC.H"

#include <iosfwd>
//using std::ostream;
#include <cstddef>
//using std::size_t;

namespace CoCoA
{

  class ConstRefRingElem; // forward declaration -- defined in ring.H
  //  class ConstRefPPMonoidElem; // forward declaration -- defined in PPMonoid.H
  class RefRingElem;      // forward declaration -- defined in ring.H
  class SparsePolyRing;   // forward declaration -- defined in SparsePolyRing.H

  class ReductionCogBase; // forward declaration for SmartPtrIRC
  class ReductionCog
  {
  public:
    explicit ReductionCog(ReductionCogBase* ptr): mySmartPtr(ptr) {}
    // assignment disabled because SmartPtrIRC has no assignment
    ReductionCogBase* operator->() { return mySmartPtr.operator->(); }  ///< Allow member fns to be called.
    const ReductionCogBase* operator->() const { return mySmartPtr.operator->(); }  ///< Allow const member fns to be called.
    bool operator==(const ReductionCog& f) const { return mySmartPtr==f.mySmartPtr; }
  private: // data members
    SmartPtrIRC<ReductionCogBase> mySmartPtr;
  };


  ConstRefPPMonoidElem ActiveLPP(const ReductionCog& F);
  bool IsActiveZero(const ReductionCog& F);
  std::ostream& operator<<(std::ostream& out, const ReductionCog& F);

  class ReductionCogBase: protected IntrusiveReferenceCount
  {
    friend class SmartPtrIRC<ReductionCogBase>; // morally "friend ReductionCog", so it can alter reference count
  protected:
    ReductionCogBase(): IntrusiveReferenceCount() {}
  public:
    virtual ~ReductionCogBase() {};
    virtual void myMoveToNextLM() =0;
    virtual void myReduce(ConstRefRingElem reducer, size_t RedLen=0) =0;
    virtual std::ostream& myOutput(std::ostream& out) const =0;
    virtual void myAssignReset(RefRingElem f) =0; // almost no-throw guarantee (?)
    virtual void myAssignReset(RefRingElem f, size_t fLen) =0; // almost no-throw guarantee (?)
    virtual void myRelease(RefRingElem f) =0; // almost no-throw guarantee (?)

    friend ConstRefPPMonoidElem ActiveLPP(const ReductionCog& F);
    friend bool IsActiveZero(const ReductionCog& F);

  protected:
    virtual bool IamActiveZero() const =0;
    virtual ConstRefPPMonoidElem myActiveLPP() const =0;
  };


  ReductionCog NewRedCogPolyField(const SparsePolyRing& P);
  ReductionCog NewRedCogPolyGCD(const SparsePolyRing& P);
  ReductionCog NewRedCogGeobucketField(const SparsePolyRing& P);
  ReductionCog NewRedCogGeobucketGCD(const SparsePolyRing& P);

  //------- inline functions ----------------------------

  inline ConstRefPPMonoidElem ActiveLPP(const ReductionCog& F)
  { return F->myActiveLPP(); }

  inline bool IsActiveZero(const ReductionCog& F)
  { return F->IamActiveZero(); }


} // end of namespace CoCoA


//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/ReductionCog.H,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: ReductionCog.H,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.8  2007/03/08 18:42:05  cocoa
// Cleaned up whitespace.
//
// Revision 1.7  2007/02/22 17:27:43  bigatti
// -- removed useless include <memory> (auto_ptr)
//
// Revision 1.6  2006/12/06 17:15:26  cocoa
// -- removed #include "config.H"
//
// Revision 1.5  2006/11/27 14:17:41  cocoa
// -- just a comment
//
// Revision 1.4  2006/11/24 17:53:14  cocoa
// -- using SmartPtrIRC for ref counting (was auto_ptr)
//
// Revision 1.3  2006/10/06 15:17:08  cocoa
// -- myReduce now returns void instead of RingElem
//
// Revision 1.2  2006/10/06 14:04:15  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.3  2006/04/11 14:16:29  cocoa
// -- reorganization of fns between reduce,SparsePolyRing,GPoly
// -- added optional "len" argument to myAssignReset in ReductionCog
//
// Revision 1.2  2006/04/10 13:38:06  cocoa
// -- added myTmpLM field to avoid repeated creations/deletions
//
// Revision 1.1  2006/03/16 14:28:29  cocoa
// -- first import
//

#endif
