#ifndef CoCoA_ServerOp_H
#define CoCoA_ServerOp_H

//   Copyright (c)  2007  John Abbott and Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "CoCoA/SmartPtrIRC.H"
#include <iosfwd>
// using std::istream;
// using std::ostream;
#include <string>
// using std::string;

namespace CoCoA
{
  //----------------------------------------------------------------------
  class ServerOpBase; // forward declaration for SmartPtrIRC

  //----------------------------------------------------------------------
  class ServerOp
  {
  public:
    explicit ServerOp(); // defined later: returns VoidOperation
    explicit ServerOp(ServerOpBase* ptr): mySmartPtr(ptr) {}
    // assignment disabled because SmartPtrIRC has no assignment
    ServerOpBase* operator->() { return mySmartPtr.operator->(); }  ///< Allow member fns to be called.
    const ServerOpBase* operator->() const { return mySmartPtr.operator->(); }  ///< Allow const member fns to be called.
    const ServerOpBase* myRawPtr() const { return mySmartPtr.myRawPtr(); } ///< Used by "downcasting" functions IsVoidOperation
    bool operator==(const ServerOp& f) const { return mySmartPtr==f.mySmartPtr; }

  private: // data members
    SmartPtrIRC<ServerOpBase> mySmartPtr;
  };


  void RegisterOp(const std::string& s, ServerOp o);
  ServerOp& GetOperation(const std::string& s);
  void CheckOperationRegistry();

  //----------------------------------------------------------------------
  class ServerOpBase: protected IntrusiveReferenceCount
  {
    friend class SmartPtrIRC<ServerOpBase>; // morally "friend ServerOp", so it can alter reference count
  protected:
    ServerOpBase() {};
  public:
    virtual ~ServerOpBase() {};
    virtual void myOutputSelf(std::ostream&) const = 0;
    virtual void myReadArgs(std::istream& in) = 0;
    virtual void myCompute() = 0;
    virtual void myWriteResult(std::ostream&) const = 0;
    virtual void myClear() = 0;

    static const std::string ourVarName4; ///< name of CoCoA4 variable which will contain the result of the computation
  };

  bool IsVoidOperation(const ServerOp& o);
  std::ostream& operator<<(std::ostream& out, const ServerOp& o);
}

// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/ServerOp.H,v 1.1 2007/04/27 14:54:22 bigatti Exp $
// $Log: ServerOp.H,v $
// Revision 1.1  2007/04/27 14:54:22  bigatti
// -- content of CoCoAServer.C split into dedicated files
// -- new registration mechanism (through include "RegisterServerOps.H")
//

#endif
