#ifndef CoCoA_random_H
#define CoCoA_random_H

//   Copyright (c)  2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <cstddef>
//using std::size_t;
#include <cstdlib>
//using std::initstate;
#include <iosfwd>
//using std::ostream;
#include <vector>
//using std::vector;

namespace CoCoA
{

  class RandomBitStream
  {
  public:
    explicit RandomBitStream(unsigned long seed=1);
    // Default copy ctor and assignment are OK.
    // Default dtor is OK.
    bool operator*() const;
    RandomBitStream& operator++();
    RandomBitStream operator++(int);
    std::size_t myIndex() const;
    void myOutputSelf(std::ostream& out) const;
  private:
    static const std::size_t ourStateBytes = 64; // read the Unix/Linux manual before changing this value!
    void myFillBuffer(); // fills myBuffer and resets myBitMask
  private: // data members
    long myBitMask;
    long myBuffer;
    std::size_t myCounter;
    const unsigned long mySeed;
    std::vector<char> myState;
  };

  bool sample(RandomBitStream& RBS);
  bool prob(double P, RandomBitStream& RBS); // returns true with probability P
  std::ostream& operator<<(std::ostream& out, const RandomBitStream& RBS);

  //---------------------------------------------------------------------------
  // Inline functions.

  inline bool RandomBitStream::operator*() const
  {
    return myBitMask&myBuffer;
  }


  inline RandomBitStream& RandomBitStream::operator++()
  {
    ++myCounter;
    myBitMask <<= 1;
    if (myBitMask <= 0) myFillBuffer();
    return *this;
  }


  inline bool sample(RandomBitStream& RBS)
  {
    return *++RBS; // advance then get value
  }

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/random.H,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: random.H,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.5  2006/12/06 17:18:40  cocoa
// -- removed #include "config.H"
//
// Revision 1.4  2006/11/27 14:19:30  cocoa
// -- added #include <cstddef> //using std::size_t;
//
// Revision 1.3  2006/10/27 19:01:17  cocoa
// Corrected a spelling mistake: Steam has become Stream :-)
//
// Revision 1.2  2006/10/06 14:04:15  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.2  2006/04/27 13:45:30  cocoa
// Changed name of NewIdentityRingHom to NewIdentityHom.
// Changed name of member functions which print out their own object
// into myOutputSelf (to distinguish from "transitive" myOutput fns).
//
// Revision 1.1  2006/04/14 13:51:40  cocoa
// Added a class for generating random bits (RandomBitStream in the file random.H).
//

#endif
