//   Copyright (c)  2007  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "CoCoA/GlobalManager.H"
#include "CoCoA/FractionField.H"

namespace CoCoA
{

  const GlobalManager* GlobalManager::ourGlobalDataPtr = 0;

  // Pseudo-ctors for RingZ and RingQ.
  ring MakeUniqueInstanceOfRingZ(); // Defined in RingZ.C.
  FractionField MakeUniqueInstanceOfRingQ(); // Defined in RingQ.C.

  GlobalManager::GlobalManager()
  {
    // Check whether a GlobalManager object has already been created, complain if so.
    if (ourGlobalDataPtr != 0)
      CoCoA_ERROR(ERR::GlobalManager2, "GlobalManager ctor");
    ourGlobalDataPtr = this; // Must happen before calling MakeUniqueInstanceOfRingQ.
    myRingZPtr.reset(new ring(MakeUniqueInstanceOfRingZ()));
    myRingQPtr.reset(new FractionField(MakeUniqueInstanceOfRingQ()));
  }


  GlobalManager::~GlobalManager()
  {
    if (ourGlobalDataPtr == 0)
      CoCoA_ERROR(ERR::SERIOUS, "GlobalManager dtor");
    ourGlobalDataPtr = 0; // "deregister" the global data
  }

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/GlobalManager.C,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: GlobalManager.C,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.2  2007/03/05 21:33:13  cocoa
// Improved/cleaned GlobalManager; added doc too.
//
// Revision 1.1  2007/03/03 14:02:11  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.1  2007/03/02 16:46:28  cocoa
// New foundations object which calls ctors and dtors of global objects.
//
