//   Copyright (c)  1997-2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <stdio.h>


#include "WGD.h"

/* This implementation is simple rather than fast.   */
/* Collins & Encarnacion explain a fast way, JSC?    */

void do_lincomb(mpz_t A, mpz_t B, int M[4])
{
  mpz_t tmp1, tmp2, tmp3;
  mpz_init(tmp1); mpz_init(tmp2); mpz_init(tmp3);
mpz_mul_ui(tmp1, A, M[0]);
mpz_mul_ui(tmp2, B, -M[1]);
mpz_mul_ui(tmp3, B, M[3]);
mpz_sub(B, tmp1, tmp2);
mpz_mul_ui(tmp2, A, -M[2]);
mpz_sub(A, tmp3, tmp2);
  mpz_clear(tmp1);  mpz_clear(tmp2);  mpz_clear(tmp3);
}

void step(int U, int V, int M[4])
{
  int qmax, qmin, Umax, Umin, Vmax, Vmin;
  
  M[0]=M[3]=1;
  M[1]=M[2]=0;
  Umin = U; Umax = U+1;
  Vmin = V; Vmax = V+1;
  while(1)
  {
    qmax = Umax/Vmin;
    qmin = Umin/Vmax;
    printf("q in [%d, %d]\n",qmin, qmax);
    if (qmax != qmin) return;
    printf("U=%d, V=%d, q=%d\n",Umin,Vmin,qmin);
    Umax -= qmin*Vmin;
    Umin -= qmin*Vmax;
    M[0] -= qmin*M[2];
    M[1] -= qmin*M[3];

    qmax = Vmax/Umin;
    qmin = Vmin/Umax;
    if (qmax != qmin) return;
    printf("U=%d, V=%d, q=%d\n",Umin,Vmin,qmin);
    Vmax -= qmin*Umin;
    Vmin -= qmin*Umax;
    M[2] -= qmin*M[0];
    M[3] -= qmin*M[1];
  }
}

int modular_to_rational(mpz_t num, mpz_t den, const mpz_t Q, const mpz_t res, const mpz_t mod)
{
  mpz_t  u_cofac, u, v_cofac, v, tmp, quot, rem, absv_cofac;
  int error;
int U,V,v_bits,u_bits,shift,M[4];
  mpz_init(v); mpz_mod(v, res, mod);

  if (mpz_sgn(v) == 0)  /* trivial case */
  {
    mpz_set(num, v);
    mpz_set_ui(den, 1);
    mpz_clear(v);
    return 0;
  }

  mpz_init(u); mpz_abs(u, mod);
  mpz_init_set_ui(u_cofac, 0);
  mpz_init_set_ui(v_cofac, 1);

  mpz_init(tmp);
  mpz_init(quot);
  mpz_init(rem);
  mpz_init_set_ui(absv_cofac, 1);

  while (mpz_sgn(v) != 0 && mpz_cmp(absv_cofac, Q) <= 0)
  {
//printf("u=");mpz_out_str(stdout,10,u);printf("\n");
//printf("v=");mpz_out_str(stdout,10,v);printf("\n");
//printf("size(u)=%d, \tsize(v)=%d\n",(int)mpz_sizeinbase(u,2),(int)mpz_sizeinbase(v,2));
u_bits = mpz_sizeinbase(u,2);
v_bits = mpz_sizeinbase(v,2);
if (u_bits > 32 && u_bits - v_bits < 15)
{
shift=u_bits-31;if (shift<0) shift=0;  
mpz_tdiv_q_2exp(tmp,u,shift);U=mpz_get_ui(tmp);
mpz_tdiv_q_2exp(tmp,v,shift);V=mpz_get_ui(tmp);
if (U < V) exit(99);
step(U,V,M);
do_lincomb(u,v,M);
do_lincomb(u_cofac, v_cofac, M);
if (mpz_sgn(u) < 0) { mpz_neg(u, u); mpz_neg(u_cofac, u_cofac); }
if (mpz_sgn(v) < 0) { mpz_neg(v, v); mpz_neg(v_cofac, v_cofac); }
if (mpz_cmp(u, v) < 0) {mpz_set(tmp,u);mpz_set(u,v);mpz_set(v,tmp);mpz_set(tmp,u_cofac);mpz_set(u_cofac,v_cofac);mpz_set(v_cofac,tmp);}
  
mpz_abs(absv_cofac, v_cofac);
}
else
{
    mpz_fdiv_qr(quot, rem, u, v);
    mpz_mul(tmp, quot, v_cofac);
    mpz_sub(tmp, u_cofac, tmp);
    mpz_set(u_cofac, v_cofac);
    mpz_set(v_cofac, tmp);
    mpz_set(u, v);
    mpz_set(v, rem);
    mpz_abs(absv_cofac, v_cofac);
}
  }

  mpz_gcd(tmp, u, u_cofac); /* this GCD is guaranteed to be positive */
  error = (mpz_cmp_ui(tmp, 1) != 0); /* non-zero if our result will be wrong */
  if (mpz_sgn(u_cofac) > 0) { mpz_set(num, u); mpz_set(den, u_cofac); }
  else                      { mpz_neg(num, u); mpz_neg(den, u_cofac); }

  mpz_clear(u_cofac);  mpz_clear(u);
  mpz_clear(v_cofac);  mpz_clear(v);
  mpz_clear(quot); mpz_clear(rem);
  mpz_clear(tmp); mpz_clear(absv_cofac);

  return error;
}
