//   Copyright (c)  2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "CoCoA/random.H"
#include "CoCoA/error.H"

#include <iostream>
using std::ostream;
#include <stdlib.h>
//using std::initstate;
//using std::setstate;
#include <time.h>
//using std::time; // to randomize automatically when the seed is 0.

namespace CoCoA
{

  // Static const data member.
  const size_t RandomBitStream::ourStateBytes;  // Just to keep the compiler quiet.


  RandomBitStream::RandomBitStream(unsigned long seed):
      myBitMask(1),
      myBuffer(0),
      myCounter(0),
      mySeed((seed==0)?time(0):seed),
      myState(ourStateBytes)
  {
    char* PrevState = initstate(mySeed, &myState[0], ourStateBytes);
//???    if (mySeed == 0) srandomdev();
    setstate(PrevState);
    myFillBuffer();
  }


  // Better define this in case some idiot wants to use it...
  // Not inline, so it is as slow as possible >-}
  RandomBitStream RandomBitStream::operator++(int)
  {
    RandomBitStream ans(*this); // Probably not cheap!
    operator++();
    return ans; // Makes another copy upon returning!
  }


  size_t RandomBitStream::myIndex() const
  {
    return myCounter;
  }


  void RandomBitStream::myOutputSelf(ostream& out) const
  {
    out << "RandomBitStream(seed=" << mySeed << ", count=" << myCounter << ")";
  }


  void RandomBitStream::myFillBuffer()
  {
    char* PrevState = setstate(&myState[0]);
    myBuffer = random();
    setstate(PrevState);
    myBitMask = 1;
  }


  // This would be cleaner if I were to use an unsigned long instead of a double.
  bool prob(double P, RandomBitStream& RBS)
  {
    if (P < 0 || P > 1) CoCoA_ERROR(ERR::BadProbability, "prob(P,RBS)");
    if (P == 1) return true;
    while (true)
    {
      if (P == 0) return false;
      bool bit = (P >= 0.5);
      if (bit ^ sample(RBS)) return bit;
      P *= 2;
      if (bit) --P;
    }
  }


  ostream& operator<<(ostream& out, const RandomBitStream& RBS)
  {
    RBS.myOutputSelf(out);
    return out;
  }

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/random.C,v 1.1.1.1 2007/03/09 15:16:11 abbott Exp $
// $Log: random.C,v $
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.2  2006/10/27 19:01:17  cocoa
// Corrected a spelling mistake: Steam has become Stream :-)
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.3  2006/04/27 13:45:30  cocoa
// Changed name of NewIdentityRingHom to NewIdentityHom.
// Changed name of member functions which print out their own object
// into myOutputSelf (to distinguish from "transitive" myOutput fns).
//
// Revision 1.2  2006/04/21 15:58:55  cocoa
// Substituted non-portable used of srandomdev (works only BSD flavours).
//
// Revision 1.1  2006/04/14 13:51:40  cocoa
// Added a class for generating random bits (RandomBitStream in the file random.H).
//
