#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingFloat.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/ZZ.H"
#include "CoCoA/io.H"

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;

using namespace CoCoA;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void TestInteger(const RefRingElem& x)
{
  ZZ n;
  if (IsInteger(n, x))
    cout << "The ring element " << x << " in " << owner(x) << " is the image of the integer " << n << endl;
  else
    cout << "The ring element " << x << " in " << owner(x) << " is not the image of an integer." << endl;
}


void trial(ring R)
{

  cout << "START OF TEST for IsInteger over " << R << endl;

  RingElem x(R, 15);
  TestInteger(x);
  if (IsDivisible(x, 2)) TestInteger(x/2);
  if (IsDivisible(x, 3)) TestInteger(x/3);
  if (IsDivisible(x, x+1)) TestInteger(x/(x+1));
  TestInteger(power(x, 10));

  cout << "END OF TEST for IsInteger over " << R << endl << endl << endl;
}

void program()
{
  GlobalManager CoCoAFoundations;

  trial(RingZ());
  trial(RingQ());
  trial(NewZmod(2));
  trial(NewZmod(3));
  trial(NewZmod(1048576)); // ring has zero divisors
  trial(NewRingFloat(256));
  try
  {
    trial(NewRingFloat(32));
  }
  catch (RingFloat::InsufficientPrecision&)
  {} // just discard InsufficientPrecision triggered by trying to convert 15^10
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
