#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingFloat.H"
#include "CoCoA/io.H"
#include "CoCoA/ring.H"

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::endl;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


void tautology(const RingElem& n)
{
  TEST_ASSERT(IsZero(n) ^ IsInvertible(n)); // RingFloat is supposed to be a field.
  ZZ N;
  TEST_ASSERT(IsInteger(N, n));
  TEST_ASSERT(n == N);
  TEST_ASSERT(N == n);
  RingElem n2(owner(n), N);
  TEST_ASSERT(n == n2);
  TEST_ASSERT(n2 == n);
}


void greater(const RingElem& n, int m)
{
  TEST_ASSERT(!(n == m));
  TEST_ASSERT(n != m);
  TEST_ASSERT(n >= m);
  TEST_ASSERT(n > m);
  TEST_ASSERT(!(n <= m));
  TEST_ASSERT(!(n < m));
  TEST_ASSERT(!(m == n));
  TEST_ASSERT(m != n);
  TEST_ASSERT(!(m >= n));
  TEST_ASSERT(!(m > n));
  TEST_ASSERT(m <= n);
  TEST_ASSERT(m < n);
}

void equal(const RingElem& n, int m)
{
  TEST_ASSERT(n == m);
  TEST_ASSERT(!(n != m));
  TEST_ASSERT(n >= m);
  TEST_ASSERT(!(n > m));
  TEST_ASSERT(n <= m);
  TEST_ASSERT(!(n < m));
  TEST_ASSERT(m == n);
  TEST_ASSERT(!(m != n));
  TEST_ASSERT(m >= n);
  TEST_ASSERT(!(m > n));
  TEST_ASSERT(m <= n);
  TEST_ASSERT(!(m < n));
}

void less(const RingElem& n, int m)
{
  TEST_ASSERT(!(n == m));
  TEST_ASSERT(n != m);
  TEST_ASSERT(!(n >= m));
  TEST_ASSERT(!(n > m));
  TEST_ASSERT(n <= m);
  TEST_ASSERT(n < m);
  TEST_ASSERT(!(m == n));
  TEST_ASSERT(m != n);
  TEST_ASSERT(m >= n);
  TEST_ASSERT(m > n);
  TEST_ASSERT(!(m <= n));
  TEST_ASSERT(!(m < n));
}


void greater(const RingElem& n, const ZZ& m)
{
  TEST_ASSERT(!(n == m));
  TEST_ASSERT(n != m);
  TEST_ASSERT(n >= m);
  TEST_ASSERT(n > m);
  TEST_ASSERT(!(n <= m));
  TEST_ASSERT(!(n < m));
  TEST_ASSERT(!(m == n));
  TEST_ASSERT(m != n);
  TEST_ASSERT(!(m >= n));
  TEST_ASSERT(!(m > n));
  TEST_ASSERT(m <= n);
  TEST_ASSERT(m < n);
}

void equal(const RingElem& n, const ZZ& m)
{
  TEST_ASSERT(n == m);
  TEST_ASSERT(!(n != m));
  TEST_ASSERT(n >= m);
  TEST_ASSERT(!(n > m));
  TEST_ASSERT(n <= m);
  TEST_ASSERT(!(n < m));
  TEST_ASSERT(m == n);
  TEST_ASSERT(!(m != n));
  TEST_ASSERT(m >= n);
  TEST_ASSERT(!(m > n));
  TEST_ASSERT(m <= n);
  TEST_ASSERT(!(m < n));
}

void less(const RingElem& n, const ZZ& m)
{
  TEST_ASSERT(!(n == m));
  TEST_ASSERT(n != m);
  TEST_ASSERT(!(n >= m));
  TEST_ASSERT(!(n > m));
  TEST_ASSERT(n <= m);
  TEST_ASSERT(n < m);
  TEST_ASSERT(!(m == n));
  TEST_ASSERT(m != n);
  TEST_ASSERT(m >= n);
  TEST_ASSERT(m > n);
  TEST_ASSERT(!(m <= n));
  TEST_ASSERT(!(m < n));
}


void arithmetic(RingElem n, const RingElem& m)
{
  ZZ N;
  TEST_ASSERT(IsInteger(N, n));
  ZZ M;
  TEST_ASSERT(IsInteger(M, m));
  TEST_ASSERT(n == N);
  TEST_ASSERT(m == M);

  TEST_ASSERT(!IsInvertible(n) || IsOne(n*(1/n)));

  // Some simple arithmetic
  n = n+m; N=N+M; TEST_ASSERT(n == N);
  n = n-m; N=N-M; TEST_ASSERT(n == N);
  n = n*m; N=N*M; TEST_ASSERT(n == N);
  n = n/m; N=N/M; TEST_ASSERT(n == N);

  n = n+M; N=N+M; TEST_ASSERT(n == N);
  n = n-M; N=N-M; TEST_ASSERT(n == N);
  n = n*M; N=N*M; TEST_ASSERT(n == N);
  n = n/M; N=N/M; TEST_ASSERT(n == N);

  n = N+m; N=N+M; TEST_ASSERT(n == N);
  n = N-m; N=N-M; TEST_ASSERT(n == N);
  n = N*m; N=N*M; TEST_ASSERT(n == N);
  n = N/m; N=N/M; TEST_ASSERT(n == N);

  // Checks for aliasing problems
  n = n+n; N=N+N; TEST_ASSERT(n == N);
  n = n*n; N=N*N; TEST_ASSERT(n == N);
  n = n/n; N=N/N; TEST_ASSERT(n == N);
  n = n-n; N=N-N; TEST_ASSERT(n == N);

  // assignment arithmetic
  N = 123456789;  n = N; TEST_ASSERT(n == N);
  n += m; N+=M; TEST_ASSERT(n == N);
  n -= m; N-=M; TEST_ASSERT(n == N);
  n *= m; N*=M; TEST_ASSERT(n == N);
  n /= m; N/=M; TEST_ASSERT(n == N);

  // Checks for aliasing problems
  n = m; n += n; N=M; N+=M; TEST_ASSERT(n == N);
  n = m; n -= n; N=M; N-=M; TEST_ASSERT(n == N);
  n = m; n *= n; N=M; N*=M; TEST_ASSERT(n == N);
  n = m; n /= n; N=M; N/=M; TEST_ASSERT(n == N);
}


void program()
{
  GlobalManager CoCoAFoundations;

  const ring RR200 = NewRingFloat(200);
  const ring RR2000 = NewRingFloat(2000);

  TEST_ASSERT(IsRingFloat(RR200));

  const ZZ Zero;

  RingElem n(RR200);

  TEST_ASSERT(IsZero(n));
  TEST_ASSERT(!IsInvertible(n));
  TEST_ASSERT(n == zero(RR200));

  equal(n, 0);
  equal(n, Zero);
  greater(n, -1);
  greater(n, ZZ(-1));
  less(n, 1);
  less(n, ZZ(1));

  n = 1;
  TEST_ASSERT(!IsZero(n));
  TEST_ASSERT(IsInvertible(n));
  TEST_ASSERT(IsOne(n));
  TEST_ASSERT(!IsMinusOne(n));
  equal(n, 1);
  equal(n, ZZ(1));
  greater(n, -1);
  greater(n, ZZ(-1));
  less(n, 2);
  less(n, ZZ(2));

  n = -1;
  TEST_ASSERT(!IsZero(n));
  TEST_ASSERT(IsInvertible(n));
  TEST_ASSERT(!IsOne(n));
  TEST_ASSERT(IsMinusOne(n));
  equal(n, -1);
  equal(n, ZZ(-1));
  greater(n, -2);
  greater(n, ZZ(-2));
  less(n, 1);
  less(n, ZZ(1));

  tautology(RingElem(RR2000, 0));
  tautology(RingElem(RR2000, 1));
  tautology(RingElem(RR2000, -1));
  tautology(RingElem(RR2000, 32003));
  tautology(RingElem(RR2000, -32003));
  tautology(RingElem(RR2000, power(ZZ(17),99)));
  tautology(RingElem(RR2000, power(ZZ(-17),99)));

  // Check correct arithmetic.  Cases are small/large, positive/negative.
  arithmetic(RingElem(RR2000, 23), RingElem(RR2000, 37));
  arithmetic(RingElem(RR2000, 100), RingElem(RR2000, power(ZZ(10), 100)));
  arithmetic(RingElem(RR2000, power(ZZ(10), 100)), RingElem(RR2000, 100));
  arithmetic(RingElem(RR2000, power(ZZ(10), 100)), RingElem(RR2000, power(ZZ(2), 301)));

  arithmetic(RingElem(RR2000, 23), -RingElem(RR2000, 37));
  arithmetic(RingElem(RR2000, 100), -RingElem(RR2000, power(ZZ(10), 100)));
  arithmetic(RingElem(RR2000, power(ZZ(10), 100)), -RingElem(RR2000, 100));
  arithmetic(RingElem(RR2000, power(ZZ(10), 100)), -RingElem(RR2000, power(ZZ(2), 301)));

  arithmetic(-RingElem(RR2000, 23), RingElem(RR2000, 37));
  arithmetic(-RingElem(RR2000, 100), RingElem(RR2000, power(ZZ(10), 100)));
  arithmetic(-RingElem(RR2000, power(ZZ(10), 100)), RingElem(RR2000, 100));
  arithmetic(-RingElem(RR2000, power(ZZ(10), 100)), RingElem(RR2000, power(ZZ(2), 301)));

  arithmetic(-RingElem(RR2000, 23), -RingElem(RR2000, 37));
  arithmetic(-RingElem(RR2000, 100), -RingElem(RR2000, power(ZZ(10), 100)));
  arithmetic(-RingElem(RR2000, power(ZZ(10), 100)), -RingElem(RR2000, 100));
  arithmetic(-RingElem(RR2000, power(ZZ(10), 100)), -RingElem(RR2000, power(ZZ(2), 301)));

  // compute a large number
  n = power(RingElem(RR200,10), 10000);
  TEST_ASSERT(n == power(ZZ(10),10000));
  TEST_ASSERT(power(RingElem(RR200,-3),3) == RingElem(RR200,-3)*RingElem(RR200,-3)*RingElem(RR200,-3));
  RingElem third = RingElem(RR200,1)/3;
  TEST_ASSERT(IsZero(3*third-1));
  ZZ TwentySeven;
  TEST_ASSERT(IsInteger(TwentySeven, power(third, -3)) && TwentySeven == 27);
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
