#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/PPMonoid.H"
#include "CoCoA/PPMonoidEv.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingFloat.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H"
#include "CoCoA/ring.H"
#include "CoCoA/symbol.H"

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::endl;
#include <vector>
using std::vector;


#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

void program()
{
  GlobalManager CoCoAFoundations;

  const ring RR = NewRingFloat(32);

  const PolyRing P = NewPolyRing(RR, symbols("x"), NewLexOrdering(1));

  RingElem x = indet(P, 0);
  RingElem one(P, 1);

  // Create two polys f and g; f divides g in exact arithmetic
  RingElem f = x - one/3;
  RingElem g = 3*x;
  g = power(g, 255) - 1;

  vector<RingElem> gens;
  gens.push_back(f);
  gens.push_back(g);
  ideal I = ideal(P, gens);

  vector<RingElem> GI = CoCoA::gens(I);
  TEST_ASSERT(GI.size() == 2);
  vector<RingElem> GBasis = TidyGens(I);
  TEST_ASSERT(GBasis.size() == 1);
  TEST_ASSERT(GBasis[0] == f);
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
