#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingFloat.H"
#include "CoCoA/RingFp.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/RingZ.H"
#include "CoCoA/io.H"

using namespace CoCoA;

#include <iostream>
using std::endl;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


// This routine deliberately amplifies any deviation from 1 in r
void IsTrulyOne(RingElem r)
{
  RingElem one(owner(r),1); one /= one;
  RingElem nine = one+one+one+one+one+one+one+one+one;
  RingElem ten = nine+one;

  for(int i=0; i <= 256; ++i)
    r = ten*r-nine;
  TEST_ASSERT(r == 1);
}


void trial(const ring& R)
{
  // We cannot conduct this test if 3 is not invertible
  if (!IsInvertible(RingElem(R,3))) return;

  RingElem one(R,1); one /= one;
  RingElem three = one+one+one;
  try
  {
    IsTrulyOne(RingElem(R,1));
  }
  catch (RingFloat::InsufficientPrecision&) {}

  try
  {
    IsTrulyOne(one);
  }
  catch (RingFloat::InsufficientPrecision&) {}

  try
  {
    IsTrulyOne(3*(one/3));
  }
  catch (RingFloat::InsufficientPrecision&) {}

  try
  {
    IsTrulyOne(three*(one/three));
  }
  catch (RingFloat::InsufficientPrecision&) {}
}


void program()
{
  GlobalManager CoCoAFoundations;

  trial(NewRingFloat(32));
  trial(NewRingFloat(256));
  trial(RingQ());
  trial(NewZmod(13));
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
