#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingFloat.H"
#include "CoCoA/io.H"

using namespace CoCoA;

#include <iostream>
using std::endl;

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


// This test unearthed a bug in GMP version 4.1.3.
// The test passes fine with GMP 4.1.4, and you are strongly
// encouraged to make sure you have this latter version or a
// still later version.

void program()
{
  GlobalManager CoCoAFoundations;

  // cout << "Program to check whether 1+delta differs from 1 for various small delta\n"
  //       << "and using various precisions of RingFloat values." << endl;
  for (int prec=32; prec <= 1024; prec *= 2)
  {
    RingFloat R = NewRingFloat(prec);
    RingElem x(R,1);
    RingElem delta(R,1);

    try
    {
      for (int i=0; i < 999; ++i)
      {
        delta /= 2;
        TEST_ASSERT(x + delta != x);
        TEST_ASSERT(x - delta != x);
      }
      TEST_ASSERT(PrecisionBits(R) > 1000);
    }
    catch (RingFloat::InsufficientPrecision&)
    {
      TEST_ASSERT(PrecisionBits(R) < 1000);
    }
  }
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(GlobalErrput());
  return 1;
}
