// Don't remember who originally reported this.
// The GBasis used to "ignore" the denominator in f.

#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingDistrMPoly.H"
#include "CoCoA/RingHom.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/ideal.H"
#include "CoCoA/io.H"

using namespace CoCoA;

#include <iostream>
using std::endl;
#include <vector>
using std::vector;

void program()
{
  GlobalManager CoCoAFoundations;

  ring Q = RingQ();
  SparsePolyRing Qx = NewPolyRing(Q, 1);
  const vector<RingElem>& x = indets(Qx);

  RingElem half(Q, 1);
  half /= 2;
  RingHom phi = CoeffEmbeddingHom(Qx);
  RingElem f(Qx);
  f = x[0] + phi(half);

  ideal I = ideal(Qx, f);
  GlobalOutput() << "f=" << f << endl;
  vector<RingElem> GBasis = TidyGens(I);
  GlobalOutput() << GBasis.front() << "\n";
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/tests/test-bug2.C,v 1.1.1.1 2007/03/09 15:16:12 abbott Exp $
// $Log: test-bug2.C,v $
// Revision 1.1.1.1  2007/03/09 15:16:12  abbott
// Imported files
//
// Revision 1.6  2007/03/03 14:13:21  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.5  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.4  2007/02/26 17:11:58  bigatti
// -- getting ready for unique ring Z: using NewZmod(N), NewRingQ()
//
// Revision 1.3  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.2  2006/12/07 17:25:32  cocoa
// -- minimal set of #include's instead of library.H
//
// Revision 1.1  2006/10/06 14:04:57  cocoa
// The new assert header and implementation files.
// A new test.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
