// Copyright (c) 2006  John Abbott
// This file is part of the CoCoALib test suite.
// You are free to use any part of this code in your own programs.

//----------------------------------------------------------------------
// WARNING: the user interface to the factorizer is not final!
// Of course, this is only a minimal test.
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

#include "CoCoA/BuildInfo.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingQ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/TmpFactor.H"
#include "CoCoA/io.H"
using namespace CoCoA;

#include <vector>
using std::vector;
#include <iostream>
using std::endl;


void program()
{
  GlobalManager CoCoAFoundations;

  ring Q = RingQ();
  SparsePolyRing P = NewPolyRing(Q,2);

  const vector<RingElem>& x = indets(P);
  RingElem f = power(x[0],96) - power(x[1],96);
  //  GlobalOutput() << "About to factorize the polynomial f = " << f << endl;
  FactorList facs = factorize(f);
  TEST_ASSERT(facs.size() == 12);
  RingElem prod = one(P);
  for (FactorList::const_iterator i = facs.begin(); i != facs.end(); ++i)
  {
    TEST_ASSERT(i->second == 1);
    prod *= i->first;
  }
  TEST_ASSERT(prod == f);
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/tests/test-factor1.C,v 1.1.1.1 2007/03/09 15:16:12 abbott Exp $
// $Log: test-factor1.C,v $
// Revision 1.1.1.1  2007/03/09 15:16:12  abbott
// Imported files
//
// Revision 1.6  2007/03/05 13:42:58  bigatti
// -- changed: no longer depends on order of factors (removed test-factor1.out)
//
// Revision 1.5  2007/03/03 14:13:21  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.4  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.3  2007/02/26 17:11:58  bigatti
// -- getting ready for unique ring Z: using NewZmod(N), NewRingQ()
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1  2006/11/11 14:51:43  cocoa
// Added simple test for the factorizer.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
